package com.aitrainer.api.test

import com.aitrainer.api.controller.CustomerExerciseDeviceController
import com.aitrainer.api.controller.PurchaseController
import com.aitrainer.api.model.Customer
import com.aitrainer.api.model.CustomerExerciseDevice
import com.aitrainer.api.repository.CustomerExerciseDeviceRepository
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.TestInstance
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.test.context.SpringBootTest
import kotlin.test.assertEquals
import kotlin.test.assertTrue

@SpringBootTest
@TestInstance(TestInstance.Lifecycle.PER_CLASS)
class CustomerExerciseDeviceTest {

    @Autowired
    private lateinit var customerExerciseDeviceRepository: CustomerExerciseDeviceRepository

    @Test
    fun testGetPurchaseByCustomerId() {
        val controller = CustomerExerciseDeviceController(customerExerciseDeviceRepository)
        val response = controller.getAllByCustomerId(90)

        val devices = response.body

        assertTrue(devices is List)
        assertTrue(devices.isNotEmpty())
        assertEquals(devices.size, 2)
        assertEquals(devices[0].exerciseDeviceId, 1)
        assertEquals(devices[0].customerId, 90)

    }


    @Test
    fun testDeviceInsert() {
       val customerExerciseDevice = CustomerExerciseDevice(
               exerciseDeviceId = 3,
               favourite = false,
               dateAdd = "2020-11-23 04:32:00",
               customerId = 62
       )


        val controller = CustomerExerciseDeviceController(customerExerciseDeviceRepository)
        val response = controller.insertDevice(customerExerciseDevice)

        val customerDeviceNew = response.body

        assertEquals(customerDeviceNew!!.customerId, 62)
        assertEquals(customerDeviceNew.exerciseDeviceId, 3)

        customerExerciseDeviceRepository.delete(customerDeviceNew)
    }

}