package com.aitrainer.api.test

import com.aitrainer.api.controller.CustomerPropertyController
import com.aitrainer.api.model.CustomerProperty
import com.aitrainer.api.repository.CustomerPropertyRepository
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.TestFactory
import org.junit.jupiter.api.TestInstance
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.test.context.SpringBootTest
import kotlin.test.assertEquals
import kotlin.test.assertTrue

@SpringBootTest
@TestInstance(TestInstance.Lifecycle.PER_CLASS)
class CustomerPropertyTest {

    @Autowired
    private lateinit var customerPropertyRepository: CustomerPropertyRepository


    @Test
    fun testGetAllCustomerProperties() {
        val controller = CustomerPropertyController(customerPropertyRepository )
        val response = controller.getAllByCustomerId(90)

        val properties = response.body

        assertTrue(properties is List)
        assertTrue(properties.isNotEmpty())
        assertEquals(properties.size, 5)
        assertEquals(properties[0].propertyValue, 81.0)
        assertEquals(properties[1].propertyValue, 82.0)
        assertEquals(properties[3].propertyId, 4)
    }

    @Test
    fun testGetLastCustomerProperties() {
        val controller = CustomerPropertyController(customerPropertyRepository)
        val response = controller.getLastPropertiesByCustomerId(90)

        val properties = response.body
        assertTrue(properties is List)
        assertTrue(properties.isNotEmpty())
        assertEquals(properties.size, 3)
        assertEquals(properties[2].propertyValue, 79.0)
        assertEquals(properties[0].propertyValue, 172.0)
    }

    @Test
    fun testInsert() {
        val customerProperty = CustomerProperty(
            customerId = 90,
            propertyId = 1,
            dateAdd = "2020-11-10",
            propertyValue = 81.5
        )

        val controller = CustomerPropertyController(customerPropertyRepository)
        val response = controller.createNewCustomerProperty(customerProperty)

        val newCustomerProperty = response.body

        assertTrue(newCustomerProperty is CustomerProperty)
        assertEquals(newCustomerProperty.propertyValue, 81.5)

        customerPropertyRepository.delete(customerProperty)
    }

}